@@ -4,12 +4,12 @@ import argparse
 import sys
 
 from datalake_scripts.scripts import add_threats, get_threats_by_hashkey, edit_score, get_threats_from_query_hash, \
-    add_comments, lookup_threats, add_tags, get_query_hash, bulk_lookup_threats
+    add_comments, lookup_threats, add_tags, get_query_hash, bulk_lookup_threats, advanced_search
 
 
 class Cli:
     CLI_NAME = 'ocd-dtl'
-    VERSION = '2.5.1'
+    VERSION = '2.5.4'
 
     def __init__(self):
         parser = argparse.ArgumentParser(
@@ -80,6 +80,10 @@ The most commonly used {self.CLI_NAME} commands are:
         args = sys.argv[2:]
         bulk_lookup_threats.main(args)
 
+    def advanced_search(self):
+        args = sys.argv[2:]
+        advanced_search.main(args)
+
     def _list_commands_available(self):
         method_list = []
         for method_name in dir(self):
