@@ -1,19 +1,21 @@
+import json
 import re
 import sys
 
 from collections import OrderedDict
-
+from halo import Halo
+from datalake import Datalake
+from datalake import OverrideType, AtomType
+from datalake.common.logger import logger
+from datalake.endpoints import Endpoint
 from datalake_scripts.common.base_script import BaseScripts
-from datalake_scripts.common.logger import logger
-from datalake_scripts.engines.post_engine import ThreatsPost, BulkThreatsPost
+from datalake_scripts.helper_scripts.utils import load_csv, load_list, save_output, parse_threat_types, flatten_list
 
 
 def main(override_args=None):
     """Method to start the script"""
-    starter = BaseScripts()
-
     # Load initial args
-    parser = starter.start('Submit a new threat to Datalake from a file')
+    parser = BaseScripts.start('Submit a new threat to Datalake from a file')
     required_named = parser.add_argument_group('required arguments')
     csv_controle = parser.add_argument_group('CSV control arguments')
     required_named.add_argument(
@@ -64,6 +66,7 @@ def main(override_args=None):
         nargs='+',
         help='choose specific threat types and their score, like: ddos 50 scam 15',
         default=[],
+        action='append',
     )
     parser.add_argument(
         '--tag',
@@ -81,6 +84,12 @@ def main(override_args=None):
         help='sets override_type to permanent. Scores won\'t be updated by the algorithm. Default is temporary',
         action='store_true',
     )
+    parser.add_argument(
+        '--lock',
+        help='sets override_type to lock. Scores won\'t be updated by the algorithm for three months. Default is '
+             'temporary',
+        action='store_true',
+    )
     parser.add_argument(
         '--no-bulk',
         help='force an api call for each threats, useful to retrieve the details of threats created',
@@ -95,50 +104,74 @@ def main(override_args=None):
     if not args.threat_types and not args.whitelist:
         parser.error("threat types is required if the atom is not for whitelisting")
 
-    permanent = 'permanent' if args.permanent else 'temporary'
+    if args.permanent and args.lock:
+        parser.error("Only one override type is authorized")
+
+    if args.permanent:
+        override_type = OverrideType.PERMANENT
+    elif args.lock:
+        override_type = OverrideType.LOCK
+    else:
+        override_type = OverrideType.TEMPORARY
 
     if args.is_csv:
         try:
-            list_new_threats = starter._load_csv(args.input, args.delimiter, args.column - 1)
+            list_new_threats = load_csv(args.input, args.delimiter, args.column - 1)
         except ValueError as ve:
             logger.error(ve)
             exit()
     else:
-        list_new_threats = starter._load_list(args.input)
+        list_new_threats = load_list(args.input)
+        if not list_new_threats:
+            raise parser.error('No atom found in the input file.')
     list_new_threats = defang_threats(list_new_threats, args.atom_type)
     list_new_threats = list(OrderedDict.fromkeys(list_new_threats))  # removing duplicates while preserving order
-    threat_types = ThreatsPost.parse_threat_types(args.threat_types) or []
+    args.threat_types = flatten_list(args.threat_types)
+    threat_types = parse_threat_types(args.threat_types)
+    atom_type = AtomType[args.atom_type.upper()]
+    dtl = Datalake(env=args.env, log_level=args.loglevel)
 
-    # Load api_endpoints and tokens
-    endpoint_config, main_url, tokens = starter.load_config(args)
+    add_threat_params = (atom_type, threat_types, override_type, args.whitelist, args.public, args.tag, args.link)
+    terminal_size = Endpoint._get_terminal_size()
     if args.no_bulk:
-        post_engine_add_threats = ThreatsPost(endpoint_config, args.env, tokens)
-        response_dict = post_engine_add_threats.add_threats(
-            list_new_threats,
-            args.atom_type,
-            args.whitelist,
-            threat_types,
-            args.public,
-            args.tag,
-            args.link,
-            permanent
-        )
+        threat_response = []
+        for threat in list_new_threats:
+            try:
+                res = dtl.Threats.add_threat(threat, *add_threat_params)
+                threat_response.append(res)
+                logger.info(f'{threat.ljust(terminal_size - 6, " ")} \x1b[0;30;42m  OK  \x1b[0m')
+            except ValueError as ve:  # Wrong atom type most likely
+                error_message = str(ve)
+                threat_response.append({"atom_value": threat, "failed": True, "error_message": error_message})
+                logger.info(f'{threat.ljust(terminal_size - 6, " ")} '
+                            f'\x1b[0;30;41m  KO  \x1b[0m')
     else:
-        post_engine_add_threats = BulkThreatsPost(endpoint_config, args.env, tokens)
-        hashkeys = post_engine_add_threats.add_bulk_threats(
-            list_new_threats,
-            args.atom_type,
-            args.whitelist,
-            threat_types,
-            args.public,
-            args.tag,
-            args.link,
-            permanent
+        spinner = Halo(text=f'Creating threats', spinner='dots')
+        spinner.start()
+        threat_response = dtl.Threats.add_threats(list_new_threats, *add_threat_params)
+        spinner.succeed()
+        failed = []
+        failed_counter = 0
+        created_counter = 0
+        for batch_res in threat_response:
+            failed.extend(batch_res['failed'])
+            for success in batch_res['success']:
+                for val_created in success['created_atom_values']:
+                    created_counter += 1
+                    logger.info(f'{val_created.ljust(terminal_size - 6, " ")} \x1b[0;30;42m  OK  \x1b[0m')
+        for failed_obj in failed:
+            for failed_atom_val in failed_obj['failed_atom_values']:
+                failed_counter += 1
+                logger.info(f'Creation failed for value {failed_atom_val.ljust(terminal_size - 6, " ")} \x1b[0;30;4\
+                1m  KO  \x1b[0m')
+        logger.info(
+            f'Number of batches: {len(threat_response)}\n'
+            f'Created threats: {created_counter}\n'
+            f'Failed threat creation: {failed_counter}'
         )
-        response_dict = {'haskeys': list(hashkeys)}
 
     if args.output:
-        starter.save_output(args.output, response_dict)
+        save_output(args.output, json.dumps(threat_response))
         logger.debug(f'Results saved in {args.output}\n')
     logger.debug(f'END: add_new_threats.py')
 
@@ -146,9 +179,9 @@ def main(override_args=None):
 def defang_threats(threats, atom_type):
     defanged = []
     # matches urls like http://www.website.com:444/file.html
-    standard_url_regex = re.compile(r'^(https?:\/\/)[a-z0-9]+([\-\.][a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$')
+    standard_url_regex = re.compile(r'^(https?://)[a-z0-9]+([\-.][a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(/.*)?$')
     # matches urls like http://185.25.5.3:8080/result.php (ipv4 or ipv6)
-    ip_url_regex = re.compile(r'^(https?:\/\/)[0-9a-zA-Z]{1,4}([\.:][0-9a-zA-Z]{1,4}){3,7}(:[0-9]{1,5})?(\/.*)?$')
+    ip_url_regex = re.compile(r'^(https?://)[0-9a-zA-Z]{1,4}([.:][0-9a-zA-Z]{1,4}){3,7}(:[0-9]{1,5})?(/.*)?$')
     for threat in threats:
         unmodified_threat = threat
         threat = threat.replace('[.]', '.')
